//=============================================================================//
//  ___  ___   _   _   _    __   _   ___ ___ __ __
// |_ _|| __| / \ | \_/ |  / _| / \ | o \ o \\ V /
//  | | | _| | o || \_/ | ( |_n| o ||   /   / \ / 
//  |_| |___||_n_||_| |_|  \__/|_n_||_|\\_|\\ |_|  2006
//										 
//=============================================================================//

#include <windows.h>
#include "common/GMLuaModule.h"

// This is a macro to make life easier.
// When the dll is loaded it will call the function (Init)
GMOD_MODULE( Init, Shutdown );


//
// A simple messagebox example
//

LUA_FUNCTION( MsgBox )
{
	// This gets the first parameter ("hello")
	const char* Msg = g_Lua->GetString(1);

	// Generic windows function to pop up a message box
	MessageBox( NULL, Msg, "Lua message!", MB_OK );

	// We're returning nothing so return 0
	return 0;
}


//
// Returning Variables
//

LUA_FUNCTION( GetVariable )
{
	// Push a string onto the Lua stack
	g_Lua->Push( "This Is Your Variable!" );

	// Return 1 because we're returning 1 string
	return 1;
}


// When the DLL is initialized it will call this function to set itself up.
// It should be used to set up your library so it's all ready to go.
int Init( void )
{
	// Msg is defined in GMLuaModule.h
	// It's actually calling the Lua function, Msg
	Msg("My Example DLL Has Been Loaded!\n");

	// Register our MsgBox function as a global
	g_Lua->SetGlobal( "MsgBox", MsgBox );

	// Lets make a global table filled with functions (a module)
	g_Lua->NewGlobalTable( "my_module" );
	ILuaObject* table = g_Lua->GetGlobal( "my_module" );

		table->SetMember( "GetVariable", GetVariable );

		// You can register the same function twice with different names
		table->SetMember( "MsgBox", MsgBox );
		table->SetMember( "GetString", GetVariable );

	// We're not deleting the table here - we're deleting the reference to it.
	// If you don't do this you're leaking memory.
	SAFE_UNREF( table );

	return 0;
}

// When the DLL is shutdown it calls this function.
// g_Lua should be assumed to have been released here
int Shutdown( void )
{
	
	return 0;
}

